"""
Reader compatibility mapping generated from test results.

This mapping shows which DatasetType readers can successfully read data
written by each (arrow_type, writer_dataset_type, content_type) combination.

Keys: (arrow_type, writer_dataset_type, content_type)
Values: List of compatible DatasetType enum values
"""

from deltacat.types.tables import DatasetType

# Mapping of (arrow_type, writer_dataset_type, content_type) -> list of compatible readers
READER_COMPATIBILITY_MAPPING = {
    ("binary", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
    ],
    ("binary", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("binary", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("bool", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date32[day]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.RAY_DATASET,
    ],
    ("date64[ms]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal128", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("decimal256", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("dictionary", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("dictionary", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("dictionary", "polars", "application/feather"): [DatasetType.POLARS],
    ("dictionary", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("dictionary", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("dictionary", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("double", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ms]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[ns]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[s]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("duration[us]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_shape_tensor", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("fixed_size_list", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("float", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("halffloat", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int16", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int32", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int64", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("int8", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
    ],
    ("large_binary", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_binary", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list_view", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list_view", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list_view", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_list_view", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("large_string", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list_view", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list_view", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list_view", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("list_view", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "polars", "application/feather"): [DatasetType.PANDAS, DatasetType.POLARS],
    ("map", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.RAY_DATASET,
    ],
    ("map", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("map", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("month_day_nano_interval", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("month_day_nano_interval", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.RAY_DATASET,
    ],
    ("null", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("null", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.RAY_DATASET,
    ],
    ("string", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("string", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
    ],
    ("struct", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("struct", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[ms]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[ms]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[ms]", "pyarrow", "application/feather"): [
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[ms]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[s]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[s]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[s]", "pyarrow", "application/feather"): [
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time32[s]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "pyarrow", "application/feather"): [
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[ns]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "pyarrow", "application/feather"): [
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("time64[us]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ms]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[ns]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[s]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pandas", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "polars", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pyarrow", "application/avro"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp[us]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ms]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[ns]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[s]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pandas", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "polars", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pyarrow", "application/orc"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("timestamp_tz[us]", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint16", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint32", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint64", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "daft", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "pandas", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "pandas", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "polars", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "polars", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "pyarrow", "application/feather"): [
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "pyarrow", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
    ("uint8", "ray_dataset", "application/parquet"): [
        DatasetType.DAFT,
        DatasetType.PANDAS,
        DatasetType.POLARS,
        DatasetType.PYARROW,
        DatasetType.RAY_DATASET,
    ],
}


def get_compatible_readers(
    arrow_type: str, writer_dataset_type: str, content_type: str
):
    """Get list of compatible reader DatasetTypes for given combination."""
    key = (arrow_type, writer_dataset_type, content_type)
    compatible_readers = READER_COMPATIBILITY_MAPPING.get(key, [])
    if (
        DatasetType.PANDAS in compatible_readers
        and DatasetType.NUMPY not in compatible_readers
    ):
        compatible_readers = compatible_readers + [DatasetType.NUMPY]
    return compatible_readers


def is_reader_compatible(
    arrow_type: str,
    writer_dataset_type: str,
    content_type: str,
    reader_dataset_type: DatasetType,
) -> bool:
    """Check if a specific reader is compatible with given combination."""
    compatible_readers = get_compatible_readers(
        arrow_type, writer_dataset_type, content_type
    )
    return reader_dataset_type in compatible_readers



# Allow classes to use self-referencing Type hints in Python 3.7.
from __future__ import annotations
from typing import Any, List, Optional
from deltacat.storage import DeltaType, LocalTable
from deltacat.compute.compactor import (
    DeltaFileEnvelope,
)
import numpy as np
import pyarrow as pa

from deltacat.compute.compactor.model.table_object_store import (
    LocalTableStorageStrategy,
    LocalTableRayObjectStoreReferenceStorageStrategy,
)


class DeleteFileEnvelope(DeltaFileEnvelope):
    @staticmethod
    def of(
        stream_position: int,
        delta_type: DeltaType,
        table: LocalTable,
        delete_columns: List[str],
        file_index: int = None,
        is_src_delta: np.bool_ = True,
        file_record_count: Optional[int] = None,
        table_storage_strategy: LocalTableStorageStrategy = LocalTableRayObjectStoreReferenceStorageStrategy(),
    ) -> DeleteFileEnvelope:
        """
        Static factory builder for a DeleteFileEnvelope. Subclasses from DeltaFileEnvelope
        `
        Args:
            stream_position: Stream position of a delta.
            delta_type: A delta type.
            table: The table object that represents the delta file.
            delete_columns: delete column_names needed for equality-based deletes,
            file_index: Manifest file index number of a delta.
            is_src_delta: True if this Delta File Locator is
                pointing to a file from the uncompacted source table, False if
                this Locator is pointing to a file in the compacted destination
                table.
            table_storage_strategy: The way the table object is stored in the delta file envelope. Defaults to LocalTableRayObjectStoreReferenceStorageStrategy
        Returns:
            A delete file envelope.

        """
        delete_file_envelope = DeltaFileEnvelope.of(
            stream_position,
            delta_type,
            table,
            file_index,
            is_src_delta,
            file_record_count,
            table_storage_strategy,
        )
        assert len(delete_columns) > 0, "At least 1 delete column is expected"
        delete_file_envelope["delete_columns"] = delete_columns
        if isinstance(table, pa.Table):
            delete_file_envelope["table_size_bytes"] = table.nbytes
        return DeleteFileEnvelope(**delete_file_envelope)

    @property
    def table_size_bytes(self) -> int:
        val = self.get("table_size_bytes")
        if val is not None:
            return val
        else:
            raise ValueError(
                f"Table type: {type(self.table)} not for supported for size method."
            )

    @property
    def delete_columns(self) -> List[str]:
        return self["delete_columns"]

    @property
    def table_reference(self) -> Optional[Any]:
        if self.table_storage_strategy is not None and isinstance(
            self.table_storage_strategy,
            LocalTableRayObjectStoreReferenceStorageStrategy,
        ):
            return self.table_storage_strategy.get_table_reference(self["table"])
        return None



from __future__ import annotations

import typing
from typing import Optional

from deltacat.storage.model.shard import Shard

T = typing.TypeVar("T")  # Type of primary key in query expression. Must be comparable


class QueryExpression(typing.Generic[T]):
    """
    Top level class for creating representing queries on a riv dataset.

    For now, this is a minimal implementation which just allows for different predicates.

    FUTURE IMPROVEMENTS
    1. Support builder using operator overloading or fluent builder pattern,e.g.
       (operator overloading) query = Column("Foo") < 10 & Column("PK")==100
       (fluent interface) query = builder.column("colA").less_than(10)
          .and_()
          .column("PK").equals(100)
          .build()

    2. Support better push down predicate integration end to end. Specifically,
        scan operation will need to return which query predicates were honored
    """

    def __init__(self):
        self.key_range: Optional[(T, T)] = None

    def with_key(self, val: T) -> QueryExpression:
        """
        Syntactic sugar for setting key range to a single value
        """
        if self.key_range:
            raise ValueError(
                f"Query expression already has set key range to: {self.key_range}"
            )
        self.key_range = (val, val)
        return self

    def with_range(self, bound1: T, bound2: T) -> QueryExpression:
        if self.key_range:
            raise ValueError(f"Key range already set to {self.key_range}")
        self.key_range = tuple(sorted([bound1, bound2]))
        return self

    @staticmethod
    def with_shard(query: Optional[QueryExpression], shard: Shard):
        """
        Generate a query expression that accounts for the shard boundaries.
        Shard boundaries are inclusive and mark the outer bounds of the query.
        """
        if shard is None:
            return query

        if query.key_range is None:
            return QueryExpression().with_range(shard.min_key, shard.max_key)

        min_key = shard.min_key
        max_key = shard.max_key

        if min_key > query.min_key:
            min_key = query.min_key

        if max_key < query.max_key:
            max_key = query.max_key

        return QueryExpression().with_range(min_key, max_key)

    @property
    def min_key(self) -> T | None:
        if not self.key_range:
            return None
        return self.key_range[0]

    @property
    def max_key(self) -> T | None:
        if not self.key_range:
            return None
        return self.key_range[1]

    def matches_query(self, key: any) -> bool:
        """
        Returns true if the key is within the range of the query expression
        """
        if not self.key_range:
            return True
        return self.min_key <= key <= self.max_key

    def below_query_range(self, key: any) -> bool:
        """
        Returns true if the key is below the range of the query expression
        will return false if key range is not set
        """
        if not self.key_range:
            return False
        return self.min_key > key



# Allow classes to use self-referencing Type hints in Python 3.7.
from __future__ import annotations
from enum import Enum
from typing import Set, Dict


class ContentType(str, Enum):
    """
    Enumeration used to resolve a file's entity-body Media Type (formerly known
    as MIME type). All content types here are writeable by at least one
    :class:`deltacat.types.media.DatasetType`. The Media Type is used as the
    content type of each :class:`deltacat.storage.model.manifest.ManifestEntry`
    written by that dataset type.

    https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17

    https://www.iana.org/assignments/media-types/media-types.xhtml
    """

    # IANA registered types
    AVRO = "application/avro"
    BINARY = "application/octet-stream"
    CSV = "text/csv"
    JSON = "application/json"

    # unregistered types
    FEATHER = "application/feather"
    ORC = "application/orc"
    PARQUET = "application/parquet"
    PSV = "text/psv"
    TSV = "text/tsv"
    UNESCAPED_TSV = "application/x-amzn-unescaped-tsv"


class ContentEncoding(str, Enum):
    """
    Enumeration used as a modifier for :class:`deltacat.types.media.ContentType`
    to indicate that additional encodings have been applied to the entity-body
    Media Type.

    https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.11

    http://www.iana.org/assignments/http-parameters/http-parameters.xhtml#content-coding
    """

    # IANA registered encodings
    GZIP = "gzip"
    DEFLATE = "deflate"
    IDENTITY = "identity"
    ZSTD = "zstd"

    # unregistered encodings
    BZIP2 = "bzip2"
    SNAPPY = "snappy"


# Map of file extensions to content types
EXT_TO_CONTENT_TYPE: Dict[str, ContentType] = {
    ".parquet": ContentType.PARQUET,
    ".pq": ContentType.PARQUET,
    ".csv": ContentType.CSV,
    ".tsv": ContentType.TSV,
    ".psv": ContentType.PSV,
    ".json": ContentType.JSON,
    ".feather": ContentType.FEATHER,
    ".avro": ContentType.AVRO,
    ".orc": ContentType.ORC,
}

# Inverse map of content types to file extensions
CONTENT_TYPE_TO_EXT: Dict[ContentType, str] = {
    v: k for k, v in EXT_TO_CONTENT_TYPE.items()
}

# Map of file extensions to content encodings
EXT_TO_CONTENT_ENCODING: Dict[str, ContentEncoding] = {
    ".gz": ContentEncoding.GZIP,
    ".bz2": ContentEncoding.BZIP2,
    ".zst": ContentEncoding.ZSTD,
    ".sz": ContentEncoding.SNAPPY,
    ".zz": ContentEncoding.DEFLATE,
    ".zip": ContentEncoding.DEFLATE,
}

# Inverse map of content encodings to file extensions
CONTENT_ENCODING_TO_EXT: Dict[ContentEncoding, str] = {
    v: k for k, v in EXT_TO_CONTENT_ENCODING.items()
}

SCHEMA_CONTENT_TYPES: Set[str] = {
    ContentType.PARQUET.value,
    ContentType.ORC.value,
    ContentType.FEATHER.value,
    ContentType.AVRO.value,
}

DELIMITED_TEXT_CONTENT_TYPES: Set[str] = {
    ContentType.UNESCAPED_TSV.value,
    ContentType.TSV.value,
    ContentType.CSV.value,
    ContentType.PSV.value,
}

TABULAR_CONTENT_TYPES: Set[str] = {
    ContentType.UNESCAPED_TSV.value,
    ContentType.TSV.value,
    ContentType.CSV.value,
    ContentType.PSV.value,
    ContentType.PARQUET.value,
    ContentType.ORC.value,
    ContentType.FEATHER.value,
    ContentType.AVRO.value,
}

EXPLICIT_COMPRESSION_CONTENT_TYPES: Set[str] = {
    ContentType.UNESCAPED_TSV.value,
    ContentType.TSV.value,
    ContentType.CSV.value,
    ContentType.PSV.value,
    ContentType.JSON.value,
}


class DatasetType(str, Enum):
    """
    Enumeration used to identify the in-memory local or distributed dataset
    to be used for file IO, queries, and data transformation. Typically used
    together with :class:`deltacat.types.media.DatastoreType` to resolve the
    compute layer that will be responsible for reading, transforming, and
    writing data to a given datastore.
    """

    # local
    NUMPY = "numpy"  # numpy.ndarray
    PANDAS = "pandas"  # pandas.DataFrame
    POLARS = "polars"  # polars.DataFrame
    PYARROW = "pyarrow"  # pyarrow.Table
    PYARROW_PARQUET = "pyarrow_parquet"  # pyarrow.parquet.ParquetFile

    # distributed
    DAFT = "daft"  # daft.DataFrame
    RAY_DATASET = "ray_dataset"  # ray.data.Dataset

    @staticmethod
    def distributed():
        return {
            DatasetType.DAFT,
            DatasetType.RAY_DATASET,
        }

    @staticmethod
    def local():
        return {
            DatasetType.NUMPY,
            DatasetType.PANDAS,
            DatasetType.POLARS,
            DatasetType.PYARROW,
            DatasetType.PYARROW_PARQUET,
        }

    def readable_content_types(self) -> Set[ContentType]:
        # if this is DAFT then it can read PARQUET, JSON, and CSV
        if self == DatasetType.DAFT:
            return {
                ContentType.PARQUET,
                ContentType.JSON,
                ContentType.CSV,
                ContentType.PSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
            }
        if self == DatasetType.RAY_DATASET:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
                ContentType.FEATHER,
            }
        if self == DatasetType.PYARROW:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.PANDAS:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.POLARS:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.NUMPY:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.PYARROW_PARQUET:
            return {ContentType.PARQUET}
        raise ValueError(f"No readable content types for {self}")

    def writable_content_types(self) -> Set[ContentType]:
        if self == DatasetType.PYARROW:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.PANDAS:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.POLARS:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.RAY_DATASET:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.JSON,
            }
        if self == DatasetType.DAFT:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.JSON,
            }
        if self == DatasetType.NUMPY:
            return {
                ContentType.CSV,
                ContentType.TSV,
                ContentType.UNESCAPED_TSV,
                ContentType.PSV,
                ContentType.PARQUET,
                ContentType.FEATHER,
                ContentType.JSON,
                ContentType.AVRO,
                ContentType.ORC,
            }
        if self == DatasetType.PYARROW_PARQUET:
            return {}
        raise ValueError(f"No writable content types for {self}")

    def can_read(self, content_type: ContentType) -> bool:
        return content_type in self.readable_content_types()

    def can_write(self, content_type: ContentType) -> bool:
        return content_type in self.writable_content_types()


# deprecated by DatasetType - populated dynamically for backwards compatibility
TableType = Enum(
    "TableType",
    {d.name: d.value for d in DatasetType.local()},
)

# deprecated by DatasetType - populated dynamically for backwards compatibility
DistributedDatasetType = Enum(
    "DistributedDatasetType",
    {d.name: d.value for d in DatasetType.distributed()},
)


# deprecated by DatasetType.local() and DatasetType.distributed()
# kept for backwards compatibility
class StorageType(str, Enum):
    LOCAL = "local"
    DISTRIBUTED = "distributed"


DATASET_TYPE_TO_SUPPORTED_READ_CONTENT_TYPES: Dict[DatasetType, Set[str]] = {
    DatasetType.DAFT: {
        ContentType.CSV,
        ContentType.PARQUET,
        ContentType.JSON,
    },
    DatasetType.RAY_DATASET: {
        ContentType.CSV,
        ContentType.TSV,
        ContentType.UNESCAPED_TSV,
        ContentType.PSV,
        ContentType.PARQUET,
        ContentType.JSON,
        ContentType.AVRO,
        ContentType.ORC,
        ContentType.FEATHER,
    },
}


class DatastoreType(str, Enum):
    """
    Enumeration used to identify the type of reader required to connect to and
    correctly interpret data stored at a given path. Typically used together
    with :class:`deltacat.types.media.DatasetType` to resolve a reader or
    writer for that data store. Note that, although some overlap exists between
    enum values here and in :class:`deltacat.types.media.ContentType`, each
    enum serve a different purpose. The purpose of
    :class:`deltacat.types.media.ContentType` is to resolve a file's MIME type,
    and may be used together with datastores that support storing different
    file types to describe the specific file type read/written from/to that
    datastore (e.g., DeltaCAT, Iceberg, Hudi, Delta Lake, Audio, Images, Video,
    etc.)
    """

    # DeltaCAT Catalog Datasets
    DELTACAT = "dc"
    DELTACAT_NAMESPACE = "namespace"
    DELTACAT_TABLE = "table"
    DELTACAT_TABLE_VERSION = "tableversion"
    DELTACAT_STREAM = "stream"
    DELTACAT_PARTITION = "partition"
    DELTACAT_DELTA = "delta"

    # External Datasets
    AUDIO = "audio"
    AVRO = "avro"
    BIGQUERY = "bigquery"
    BINARY = "binary"
    CSV = "csv"
    CLICKHOUSE = "clickhouse"
    DATABRICKS_TABLES = "databricks"
    DELTA_LAKE = "deltalake"
    DELTA_SHARING = "deltasharing"
    FEATHER = "feather"
    HDF = "hdf"
    HTML = "html"
    HUDI = "hudi"
    ICEBERG = "iceberg"
    IMAGES = "images"
    JSON = "json"
    LANCE = "lance"
    MONGO = "mongodb"
    NUMPY = "numpy"
    ORC = "orc"
    PARQUET = "parquet"
    TEXT = "text"
    TFRECORDS = "tfrecords"
    VIDEOS = "videos"
    WARC = "warc"
    WEBDATASET = "webdataset"
    XML = "xml"

    def url(self, url: str) -> str:
        """
        Returns a DeltaCAT URL string for this datastore type and the given base URL.
        Typically, DeltaCAT URLs will be of the form <DatastoreType>+<URL>.

        However, the following Datastore Types don't follow the <DatastoreType>+<URL> convention:

        {DatastoreType.MONGO}: <mongodb_uri>?database=<db_name>&collection=<collection_name>&...
        {DatastoreType.BIGQUERY}: bigquery://<project_id>/<dataset>?param1=val1&...
        {DatastoreType.CLICKHOUSE}: <clickhouse_dsn>?table=<table_name>?param1=val1&...
        {DatastoreType.DATABRICKS_TABLES}: databricks://<warehouse_id>?param1=val1&...
        {DatastoreType.ICEBERG}: iceberg://<table_identifier>?param1=val1&...

        Args:
            url: The base URL to convert to a DeltaCAT URL.

        Returns:
            A DeltaCAT URL string for this datastore type and the given URL.
        """
        if self == DatastoreType.BIGQUERY:
            raise ValueError(
                f"No DataStore URL for BigQuery. Use a URL of the form: bigquery://<project_id>/<dataset>?param1=val1&..."
            )
        if self == DatastoreType.CLICKHOUSE:
            raise ValueError(
                f"No DataStore URL for ClickHouse. Use a URL of the form: <clickhouse_dsn>?table=<table_name>?param1=val1&..."
            )
        if self == DatastoreType.DATABRICKS_TABLES:
            raise ValueError(
                f"No DataStore URL for Databricks. Use a URL of the form: databricks://<warehouse_id>?param1=val1&..."
            )
        if self == DatastoreType.ICEBERG:
            raise ValueError(
                f"No DataStore URL for Iceberg. Use a URL of the form: iceberg://<table_identifier>?param1=val1&..."
            )
        if self == DatastoreType.MONGO:
            raise ValueError(
                f"No DataStore URL for MongoDB. Use a URL of the form: <mongodb_uri>?database=<db_name>&collection=<collection_name>&..."
            )
        if self in [
            DatastoreType.DELTACAT,
            DatastoreType.DELTACAT_NAMESPACE,
            DatastoreType.DELTACAT_TABLE,
            DatastoreType.DELTACAT_TABLE_VERSION,
            DatastoreType.DELTACAT_STREAM,
            DatastoreType.DELTACAT_PARTITION,
            DatastoreType.DELTACAT_DELTA,
        ]:
            raise ValueError(
                f"No DataStore URL for DeltaCAT. Use a URL of the form: dc://<catalog>/[namespace]/[table]/[tableversion]/[stream]/[partition]/[delta]"
            )
        return f"{self.value}+{url}"

    @staticmethod
    def from_url(url: str) -> DatastoreType:
        """
        Returns an inferred DatastoreType for the given URL.

        Args:
            url: The URL or file path to analyze for datastore type inference.

        Returns:
            An inferred DatastoreType for the given URL.

        Raises:
            ValueError: If a DatastoreType cannot be inferred from the given URL.
        """
        # Detect by prefix first
        # DeltaCAT URLs
        if url.startswith("dc://"):
            return DatastoreType.DELTACAT

        # External Datastore Types
        if url.startswith("hudi+") or url.startswith("hudi://"):
            return DatastoreType.HUDI
        if url.startswith("iceberg+") or url.startswith("iceberg://"):
            return DatastoreType.ICEBERG
        if url.startswith("deltalake+") or url.startswith("deltalake://"):
            return DatastoreType.DELTA_LAKE
        if url.startswith("deltasharing+") or url.startswith("deltasharing://"):
            return DatastoreType.DELTA_SHARING
        if url.startswith("bigquery+") or url.startswith("bigquery://"):
            return DatastoreType.BIGQUERY
        if url.startswith("clickhouse+") or url.startswith("clickhouse://"):
            return DatastoreType.CLICKHOUSE
        if url.startswith("databricks+") or url.startswith("databricks://"):
            return DatastoreType.DATABRICKS_TABLES
        if url.startswith("mongodb+") or url.startswith("mongodb://"):
            return DatastoreType.MONGO

        # File Format Types
        if url.startswith("binary+") or url.startswith("binary://"):
            return DatastoreType.BINARY
        if url.startswith("csv+") or url.startswith("csv://"):
            return DatastoreType.CSV
        if url.startswith("json+") or url.startswith("json://"):
            return DatastoreType.JSON
        if url.startswith("avro+") or url.startswith("avro://"):
            return DatastoreType.AVRO
        if url.startswith("orc+") or url.startswith("orc://"):
            return DatastoreType.ORC
        if url.startswith("feather+") or url.startswith("feather://"):
            return DatastoreType.FEATHER
        if url.startswith("numpy+") or url.startswith("numpy://"):
            return DatastoreType.NUMPY
        if url.startswith("parquet+") or url.startswith("parquet://"):
            return DatastoreType.PARQUET
        if url.startswith("hdf+") or url.startswith("hdf://"):
            return DatastoreType.HDF
        if url.startswith("lance+") or url.startswith("lance://"):
            return DatastoreType.LANCE
        if url.startswith("tfrecords+") or url.startswith("tfrecords://"):
            return DatastoreType.TFRECORDS
        if url.startswith("webdataset+") or url.startswith("webdataset://"):
            return DatastoreType.WEBDATASET

        # Text and Web Types
        if url.startswith("text+") or url.startswith("text://"):
            return DatastoreType.TEXT
        if url.startswith("html+") or url.startswith("html://"):
            return DatastoreType.HTML
        if url.startswith("warc+") or url.startswith("warc://"):
            return DatastoreType.WARC
        if url.startswith("xml+") or url.startswith("xml://"):
            return DatastoreType.XML

        # Media Types
        if url.startswith("audio+") or url.startswith("audio://"):
            return DatastoreType.AUDIO
        if url.startswith("images+") or url.startswith("images://"):
            return DatastoreType.IMAGES
        if url.startswith("videos+") or url.startswith("videos://"):
            return DatastoreType.VIDEOS

        extension = "." + url.split(".")[-1].lower()

        # Fallback to file-extensions
        if extension in [".parquet", ".pq"]:
            return DatastoreType.PARQUET
        if extension == ".csv":
            return DatastoreType.CSV
        if extension == ".json":
            return DatastoreType.JSON
        if extension == ".avro":
            return DatastoreType.AVRO
        if extension == ".orc":
            return DatastoreType.ORC
        if extension == ".feather":
            return DatastoreType.FEATHER
        if extension == ".npy":
            return DatastoreType.NUMPY

        # Text formats
        if extension in [".txt", ".text", ".md"]:
            return DatastoreType.TEXT

        # Data science formats
        if extension in [".hdf", ".h5", ".hdf5"]:
            return DatastoreType.HDF
        if extension == ".lance":
            return DatastoreType.LANCE
        if extension in [".tfrecords", ".tfrecord"]:
            return DatastoreType.TFRECORDS
        if extension == ".webdataset":
            return DatastoreType.WEBDATASET

        # Web formats
        if extension in [".html", ".htm"]:
            return DatastoreType.HTML
        if extension == ".warc":
            return DatastoreType.WARC
        if extension == ".xml":
            return DatastoreType.XML

        # Binary formats
        if extension in [".bin", ".exe", ".dll", ".so", ".dylib", ".a", ".lib"]:
            return DatastoreType.BINARY

        # Media formats - Images
        if extension in [
            ".jpg",
            ".jpeg",
            ".png",
            ".gif",
            ".bmp",
            ".tiff",
            ".tif",
            ".ico",
            ".webp",
            ".svg",
            ".heic",
            ".heif",
            ".jp2",
            ".jfif",
            ".pjpeg",
            ".pjp",
        ]:
            return DatastoreType.IMAGES

        # Media formats - Videos
        if extension in [
            ".mp4",
            ".mov",
            ".avi",
            ".mkv",
            ".webm",
            ".flv",
            ".wmv",
            ".m4v",
            ".3gp",
            ".3g2",
            ".f4v",
            ".asf",
            ".rm",
            ".rmvb",
            ".vob",
            ".ogv",
            ".drc",
            ".mng",
            ".qt",
            ".yuv",
            ".mpg",
            ".mpeg",
            ".m2v",
            ".m2ts",
            ".mts",
            ".ts",
        ]:
            return DatastoreType.VIDEOS

        # Media formats - Audio
        if extension in [
            ".mp3",
            ".wav",
            ".ogg",
            ".flac",
            ".aac",
            ".m4a",
            ".m4b",
            ".m4p",
            ".wma",
            ".ra",
            ".amr",
            ".ape",
            ".au",
            ".gsm",
            ".dss",
            ".dvf",
            ".msv",
            ".opus",
            ".tta",
            ".voc",
            ".vox",
            ".wv",
            ".3ga",
            ".ac3",
            ".adt",
            ".adts",
        ]:
            return DatastoreType.AUDIO

        # Default to binary
        return DatastoreType.BINARY

    @staticmethod
    def get_url(url: str) -> str:
        """
        Returns a DeltaCAT URL string with an inferred datastore type for the given URL.

        Args:
            url: The URL or file path to analyze for datastore type inference.

        Returns:
            A DeltaCAT URL string for the inferred datastore type.

        Raises:
            ValueError: If a DeltaCAT URL cannot be inferred from the given URL.
        """
        return DatastoreType.from_url(url).url(url)
